@@ -104,19 +104,6 @@ class Mlx5MkeyTest(RDMATestCase):
         self.server = None
         self.client = None
 
-    def create_players(self, resource, **resource_arg):
-        """
-        Init Mkey test resources.
-        :param resource: The RDMA resources to use.
-        :param resource_arg: Dict of args that specify the resource specific
-                             attributes.
-        :return: None
-        """
-        self.client = resource(**self.dev_info, **resource_arg)
-        self.server = resource(**self.dev_info, **resource_arg)
-        self.client.pre_run(self.server.psns, self.server.qps_num)
-        self.server.pre_run(self.client.psns, self.client.qps_num)
-
     def reg_mr_list(self, configure_mkey=False):
         """
         Register a list of SGEs using the player's mkeys.
@@ -173,9 +160,7 @@ class Mlx5MkeyTest(RDMATestCase):
             player.qp.wr_set_mkey_access_flags(e.IBV_ACCESS_LOCAL_WRITE)
             player.qp.wr_set_mkey_layout_list([sge])
 
-            t10dif_flags = (dve.MLX5DV_SIG_T10DIF_FLAG_REF_REMAP |
-                            dve.MLX5DV_SIG_T10DIF_FLAG_APP_ESCAPE |
-                            dve.MLX5DV_SIG_T10DIF_FLAG_APP_REF_ESCAPE)
+            t10dif_flags = dve.MLX5DV_SIG_T10DIF_FLAG_REF_REMAP
             sig_t10dif = Mlx5SigT10Dif(bg_type=dve.MLX5DV_SIG_T10DIF_CRC,
                                        bg=0xFFFF, app_tag=0xABCD,
                                        ref_tag=0x01234567, flags=t10dif_flags)
